#include "fxlib.h"
#include "platform.h"
#include "linker.h"
#include "proc.h"
#include "graph.h"
#include "buffer.h"
#include "string.h"
#include "getkey.h"
#include "event.h"
#include "assembler.h"

//menu options
#define OPTION_EXEC 0
#define OPTION_ASM 1
#define OPTION_MAX OPTION_ASM

//key modes
#define KEY_MODE_NORMAL 0
#define KEY_MODE_UPPER 1
#define KEY_MODE_LOWER 2

//useful macros
#define print(a) Print((unsigned char*)(a))
#define clear() Graph_fill(COLOR_WHITE)
#define text(x, y, a) do {locate((x), (y)); print((a));} while(0)
#define halt() do{unsigned int i; while(1) GetKey(&i);} while(1)

//main menu
int menu() {
	int key, redraw=1;
	int option=0;
	
	//draw the menu on screen
	clear();
	text(9, 1, "ASM");
	text(2, 3, "Execute script");
	text(2, 4, "Assemble script");
	
	//menu loop
	while(1) {
		if(redraw) {
			//clear the cursor on all options
			for(int i=0; i<=OPTION_MAX; i++) text(1, i+3, " ");
			//draw the cursor
			text(1, option+3, ">");
			//print the VRAM
			Graph_printVram();
			
			redraw=0;
		}
		
		//read the key
		key=getKey();
		text(5, 6, "        ");
		text(5, 6, itodec(key));
		switch(key) {
			case 37: //down
				if(option!=OPTION_MAX) option++;
				redraw=1;
				break;
			case 28: //up
				if(option!=0) option--;
				redraw=1;
				break;
			case 31: //exe
				return option;
		}
	}
}

//routine to read some text from the user
char text[16];
char* getText(char* prompt, int length) {
	int key, mode=KEY_MODE_NORMAL;
	int redraw=1;
	int pos=0;
	
	//clear the string
	for(int i=0; i<16; i++) text[i]='\0';
	
	//draw the menu on screen
	clear();
	text((21-strlend(prompt))/2, 1, prompt);
	text(1, 2, "Input mode:");
	text(1, 3, "[");
	text(2+length, 3, "]");
	
	//text input loop
	while(1) {
		if(redraw) {
			//print the input mode
			text(13, 2, "         ");
			if(mode==KEY_MODE_NORMAL) text(13, 2, "Numeric");
			else if(mode==KEY_MODE_LOWER) text(13, 2, "Lowercase");
			else text(13, 2, "Uppercase");
			
			//redraw the text
			for(int i=0; i<length; i++) text(2+i, 3, " ");
			text(2, 3, text);
			
			//remove the position indicator
			for(int i=0; i<length; i++) text(2+i, 4, " ");
			
			//draw the position indicator at the correct spot
			text(2+pos, 4, "^");
			
			//display the VRAM to screen
			Graph_printVram();
			
			redraw=0;
		}
		
		//read the key
		key=getKey();
		
		//switch the input mode if the correct key is pressed
		if(key==77) { //ALPHA
			if(mode==KEY_MODE_NORMAL) mode=KEY_MODE_LOWER;
			else mode=KEY_MODE_NORMAL;
			redraw=1;
		} else if(key==78) { //SHIFT
			if(mode==KEY_MODE_LOWER) mode=KEY_MODE_UPPER;
			else if(mode==KEY_MODE_UPPER) mode=KEY_MODE_LOWER;
			redraw=1;
		} else if(key==44) { //DEL
			if(pos) {
				text[--pos]='\0';
				redraw=1;
			}
		} else if(key==31) { //EXE
			if(pos) {
				return text;
			}
		} else { //read a character
			
			//make sure we don't overflow
			if(pos==length) continue;
			
			if(mode==KEY_MODE_NORMAL) { //a number
				redraw=1;
				switch(key) {
					case 71:
						text[pos++]='0';
						break;
					case 72:
						text[pos++]='1';
						break;
					case 62:
						text[pos++]='2';
						break;
					case 52:
						text[pos++]='3';
						break;
					case 73:
						text[pos++]='4';
						break;
					case 63:
						text[pos++]='5';
						break;
					case 53:
						text[pos++]='6';
						break;
					case 74:
						text[pos++]='7';
						break;
					case 64:
						text[pos++]='8';
						break;
					case 54:
						text[pos++]='9';
						break;
					case 61:
						text[pos++]='.';
						break;
					case 32:
						text[pos++]='-';
						break;
					default:
						redraw=0;
				}
			} else { //a letter
				char letter='\0';
				switch(key) {
					case 76:
						letter='A';
						break;
					case 66:
						letter='B';
						break;
					case 56:
						letter='C';
						break;
					case 46:
						letter='D';
						break;
					case 36:
						letter='E';
						break;
					case 26:
						letter='F';
						break;
					case 75:
						letter='G';
						break;
					case 65:
						letter='H';
						break;
					case 55:
						letter='I';
						break;
					case 45:
						letter='J';
						break;
					case 35:
						letter='K';
						break;
					case 25:
						letter='L';
						break;
					case 74:
						letter='M';
						break;
					case 64:
						letter='N';
						break;
					case 54:
						letter='O';
						break;
					case 73:
						letter='P';
						break;
					case 63:
						letter='Q';
						break;
					case 53:
						letter='R';
						break;
					case 43:
						letter='S';
						break;
					case 33:
						letter='T';
						break;
					case 72:
						letter='U';
						break;
					case 62:
						letter='V';
						break;
					case 52:
						letter='W';
						break;
					case 42:
						letter='X';
						break;
					case 32:
						letter='Y';
						break;
					case 71:
						letter='Z';
						break;
					case 61:
						letter=' ';
						break;
				}
				if(letter) {
					redraw=1;
					if(mode==KEY_MODE_LOWER&&(letter>='A'&&letter<='Z')) {
						letter+=32;
					}
					text[pos++]=letter;
				}
			}
		}
	}
}

//assembler menu
void menuAsm() {
	char inFile[16];
	char outFile[16];
	char optFile[16];
	int hasOpt=0;
	
	//read input and output files
	strcpyd(getText("Input file", 15), inFile);
	strcpyd(getText("Output file", 15), outFile);
	
	//ask the user if we need an option file
	clear();
	text(1, 1, "Use an option file?");
	text(1, 2, "F1: yes");
	text(1, 3, "F2: no");
	Graph_printVram();
	while(1) {
		int key=getKey();
		if(key==79) {
			hasOpt=1;
			break;
		} else if(key==69) {
			break;
		}
	}
	if(hasOpt) strcpyd(getText("Option file", 15), optFile);
	
	//display it to the user
	clear();
	text(1, 1, "Input:");
	text(1, 2, "Output:");
	text(1, 3, "Option:");
	text(9, 1, inFile);
	text(9, 2, outFile);
	if(hasOpt) text(9, 3, optFile);
	else text(9, 3, "None");
	text(1, 4, "Is this OK?");
	text(1, 5, "F1: yes");
	text(1, 6, "F2: no");
	Graph_printVram();
	
	//await user response
	while(1) {
		int key=getKey();
		if(key==79) break;
		if(key==69) return;
	}
	
	//actually assemble the file
	clear();
	text(1, 1, "Working...");
	Graph_printVram();
	buffer_t *info;
	Platform_init();
	int status=Assembler_doFile(inFile, outFile, hasOpt?optFile:0, &info);
	Platform_quit();
	
	//print result
	clear();
	if(!status) text(1, 1, "OK");
	else {
		text(1, 1, "Error:");
		text(8, 1, itodec(status));
	}
	text(1, 2, "F1: log to file");
	text(1, 3, "F2: exit");
	Graph_printVram();
	
	//await user response
	while(1) {
		int key=getKey();
		if(key==79) break;
		if(key==69) {
			freeBuffer(info);
			return;
		}
	}
	
	//get log file
	char* logFile=getText("Log file", 15);
	int fd=File_open(logFile, FILE_OPEN_write|FILE_OPEN_create);
	if(fd<0) {
		clear();
		text(1, 1, "Couldn't open log file");
		text(1, 2, "Error:");
		text(8, 2, itodec(fd));
		Graph_printVram();
		freeBuffer(info);
		halt();
	}
	
	//print to file
	int len=File_write(fd, 0, info->data, info->size);
	if(len!=info->size||File_truncate(fd, info->size)) {
		clear();
		text(1, 1, "Error writing to file");
		text(1, 2, itodec(len));
		text(1, 3, itodec(info->size));
		Graph_printVram();
		freeBuffer(info);
		halt();
	}
	freeBuffer(info);
	
	//close file
	int close=File_close(fd);
	if(close) {
		clear();
		text(1, 1, "Error closing file");
		text(1, 2, itodec(close));
		text(1, 3, itodec(len));
		text(1, 4, itodec(File_length(fd)));
		Graph_printVram();
		halt();
	}
	
	//print the success
	clear();
	text(1, 1, "Successfully written to disk");
	text(1, 2, "File:");
	text(6, 2, logFile);
	Graph_printVram();
	
	getKey();
}

//exec menu
void menuExec() {
	proc_t proc;
	int ok;
	
	//get filename
	getText("Execute file", 15);
	clear();
	Graph_printVram();
	
	//init platform
	Platform_init();
	
	//init the processor
	initProc(&proc);
	ok=Linker_link(text, &proc);
	
	//clear the event queue
	Event_clean();
	
	if(ok) {
		//execute the program
		Proc_execute(&proc, -1);
		
		//unmap the segments
		Segments_unmapAll(&proc.mmu);
		
		
		//exit platform
		Platform_quit();
	} else {
		//exit platform
		Platform_quit();
		
		//display an error message
		int err=Platform_getError();
		clear();
		text(1, 1, "Error code:");
		text(13, 1, itodec(err));
		halt();
	}
}

int main(int isAppli, unsigned short OptionNum) {
	while(1) {
		int option=menu();
		if(option==OPTION_ASM) {
			menuAsm();
		} else if(option==OPTION_EXEC) {
			menuExec();
		}
	}
	halt();
	return 1;
}
